import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, ItemDurabilityComponent } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

const settingsOrder = [
  "notifyDamagePercent",
  "visualDurabilities",
];

const defaultValues = {
  notifyDamagePercent: 5,
  visualDurabilities: 1,
};

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_visual_durabilities:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_visual_durabilities:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

if (world.scoreboard.getObjective("settings") == undefined) {
  world.scoreboard.addObjective("settings", "settings");
}

settingsOrder.forEach((setting) => {
  const objective = world.scoreboard.getObjective("settings");
  try {
    objective.getScore(setting);
  } catch (error) {
    // Set the default value if an error occurs
    const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
    objective.setScore(setting, defaultValue);
  }
});


system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;
    if (!Boolean(world.scoreboard.getObjective("settings").getScore("visualDurabilities"))) return;

    // Iterate through each slot in the player's inventory
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;

      // Check if the item has a durability component
      if (itemStack.hasComponent(ItemDurabilityComponent.componentId)) {
        const durabilityComponent = itemStack.getComponent(ItemDurabilityComponent.componentId);
        const currentDurability = durabilityComponent.damage;
        const maxDurability = durabilityComponent.maxDurability;

        // Create a lore line showing durability
        const durabilityLore = `§r§7Durability: ${
          maxDurability - currentDurability + 1
        }/${maxDurability + 1}`;
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array

        // Remove existing durability lore (if any) and add the new one
        const updatedLore = lore.filter(
          (line) => !line.startsWith("§r§7Durability:")
        );
        updatedLore.push(durabilityLore);

        // Check if the tool's durability is at or below 5%
        const remainingDurability = maxDurability - currentDurability + 1;
        const alertLore = "§r§cLow Durability!§r"; // Special lore to indicate alert sent

        if (
          remainingDurability <=
          Math.ceil(maxDurability * (world.scoreboard.getObjective("settings").getScore("notifyDamagePercent")/100))
        ) {
          // Check if the alert was already sent
          if (!lore.includes(alertLore)) {
            player.sendMessage(
              `§cYour tool has ${remainingDurability}/${
                maxDurability + 1
              } durability remaining!`
            );
            player.playSound("note.bass", {
              pitch: 2,
              volume: 1,
              location: player.location,
            });
            updatedLore.push(alertLore); // Add the alert lore
          }
        } else if (remainingDurability > Math.ceil(maxDurability * (world.scoreboard.getObjective("settings").getScore("notifyDamagePercent")/100))) {
          // Remove the alert lore if durability is above 10%
          const alertIndex = updatedLore.indexOf(alertLore);
          if (alertIndex !== -1) {
            updatedLore.splice(alertIndex, 1); // Remove the alert lore
          }
        }

        // Set the updated lore back to the item
        itemStack.setLore(updatedLore);
        container.setItem(i, itemStack);
      }
    }
  }
});



world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_visual_durabilities_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_visual_durabilities_version", VERSION);
    if (player.hasTag("sg_visual_durabilities_received_guide")) {
      player.removeTag("sg_visual_durabilities_received_guide");
      player.sendMessage({
        translate: "sg_visual_durabilities.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_visual_durabilities:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_visual_durabilities_version", VERSION);
    player.sendMessage({
      translate: "sg_visual_durabilities.announce_update",
      with: [VERSION],
    });
  }
});

const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_visual_durabilities.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {
      const objective = world.scoreboard.getObjective("settings");

      // Create and show the modal form
      const form = new ModalFormData()
        .title("Options")
        .slider(`Alert at _%% damage`, 1, 50, 1, objective.getScore("notifyDamagePercent"))
        .toggle("Enabled", Boolean(objective.getScore("visualDurabilities")))
        .submitButton("Save Settings");

      form.show(player).then((res) => {
        if (res.canceled) return;

        // Iterate over form values and map them to the settings labels
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_visual_durabilities.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_visual_durabilities.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_visual_durabilities.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_visual_durabilities.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(
        player,
        "sg_visual_durabilities.guide.changelog.title",
        body.build(),
        [
          {
            translation_key: "sg_visual_durabilities.guide.common.back_button",
            action: (player) => showMainPage(player),
          },
        ]
      );
    },
  },
  {
    translation_key: "sg_visual_durabilities.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_visual_durabilities:guide") {
      showMainPage(event.source);
    }
  });
});